@@ -1,8 +1,11 @@
+# vim: set ft=perl ts=8 sts=2 sw=2 tw=100 et :
 use strictures 2;
 # no package, so things defined here appear in the namespace of the parent.
 
 use 5.020;
-use experimental qw(signatures postderef);
+use stable 0.031 'postderef';
+use experimental 'signatures';
+use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
